function [U0,U1] = subspaceit(U,use_rand,U1)
%SUBSPACEIT   Subspace iteration for computing invariant subspace.
%   [U0,U1] = SUBSPACEIT(U,use_rand,U1) computes an orthogonal basis U0 
%   for the column space of the square matrix U.
%   Normally one or two steps will yield convergence.
%   U1 is the orthogonal complement of U0.
%   Optional inputs:
%     use_rand: 1 to use randomization to form initial matrix (default 0).
%     U1: initial matrix (then use_rand becomes irrelevant).

n = length(U);
xsize = round(norm(U,'fro')^2); % (Accurate) estimate of norm of U0.

if nargin < 2; use_rand = 0; end

% Determine initial matrix.
if nargin >= 3 % Initial guess given.
    UU = U*U1;
elseif use_rand % Random initial guess.
    UU = U*randn(n,min(xsize+3,n));
else % Take large columns of U as initial guess.

normcols = zeros(1,n);
for ii = 1:n; normcols(ii) = norm(U(:,ii)); end;
[normc,IX] = sort(normcols,'descend');
UU = U(:,IX(1:min(xsize+3,n))); % Take columns of large norm.

% or 
%UU = U(:,1:min(xsize+3,n));     % Take first columns.
end

[UU,R] = qr(UU,0); UU = U*UU;[UU,R] = qr(UU); % Subspace iteration.
U0 = UU(:,1:xsize); U1 = UU(:,xsize+1:end);